package de.fhdw.gaming.ipspiel23.memory;


/**
 * Implements {@link IGameMemoryCapacity}.
 */
public final class GameMemoryCapacity implements IGameMemoryCapacity {

    /**
     * True when the default capacity is used. Currently 5.
     * Have to call it "v" because of pmd.
     */
    private final boolean vDefault;
    
    /**
     * True when the capacity is unlimited.
     * Have to call it "v" because of pmd.
     */
    private final boolean vUnlimited;
    
    /**
     * Capacity of the memory. -1 if it's unlimited.
     */
    private final int capacity;
    
    /**
     * private Constructor.
     * @param isDefault default.
     * @param isUnlimited unlimited capacity.
     * @param capacity Custom capacity.
     */
    private GameMemoryCapacity(final boolean isDefault, final boolean isUnlimited, final int capacity) {
        this.vDefault = isDefault;
        this.vUnlimited = isUnlimited;
        this.capacity = capacity;
    }

    /**
     * unlimited memory capacity.
     */
    public static IGameMemoryCapacity unlimited() {
        return new GameMemoryCapacity(false, true, -1);
    }

    /**
     * use the specified capacity.
     * @param capacity t.
     */
    public static IGameMemoryCapacity of(final Integer capacity) {
        if (capacity < 0) {
            throw new IllegalArgumentException("Capacity must be greater than or equal to 0");
        }
        return new GameMemoryCapacity(false, false, capacity);
    }

    /**
     * Uses a default capacity. Default is capacity of 5.
     */
    public static IGameMemoryCapacity useDefault() {
        return new GameMemoryCapacity(true, false, 5);
    }

    @Override
    public boolean isUnlimited() {
        return this.vUnlimited;
    }

    @Override
    public boolean isDefault() {
        return this.vDefault;
    }

    @Override
    public int getCapacity() {
        return this.capacity;
    }
}
